@@ -15,6 +15,7 @@
 # ==============================================================================
 """Chex assertion utilities."""
 import collections
+import collections.abc
 import functools
 import inspect
 import itertools
@@ -22,26 +23,26 @@ import traceback
 from typing import Any, Sequence, Type, Union, Callable, Optional, Set
 import unittest
 from unittest import mock
+
 from chex._src import pytypes
 import jax
 import jax.numpy as jnp
 import jax.test_util as jax_test
 import numpy as np
-import tree
+import tree as dm_tree
 
 Scalar = pytypes.Scalar
 Array = pytypes.Array
 ArrayTree = pytypes.ArrayTree
 
 # Custom pytypes.
-TIndex = int
 TLeaf = Any
 TLeavesEqCmpFn = Callable[[TLeaf, TLeaf], bool]
 TLeavesEqCmpErrorFn = Callable[[TLeaf, TLeaf], str]
 
 
-def _num_devices_available(devtype: str, backend: Optional[str] = None):
-  """Returns number of available device of the given type."""
+def _num_devices_available(devtype: str, backend: Optional[str] = None) -> int:
+  """Returns the number of available device of the given type."""
   devtype = devtype.lower()
   supported_types = ("cpu", "gpu", "tpu")
   if devtype not in supported_types:
@@ -79,13 +80,20 @@ def _is_traceable(fn):
     if any(t in str(fn_) for t in tokens):
       return True
 
-    if hasattr(fn_, "__wrapped__") and getattr(fn_, "__globals__", {}).get(
-        "__name__", None) == "jax.api":
-      # Wrapper from `jax.api`.
-      return True
+    if hasattr(fn_, "__wrapped__"):
+      # Wrapper.
+      fn_globals = getattr(fn_, "__globals__", {})
 
-    if not hasattr(fn_, "__wrapped__"):
+      if fn_globals.get("__name__", None) == "jax.api":
+        # Wrapper from `jax.api`.
+        return True
+
+      if "api_boundary" in fn_globals:
+        # api_boundary is a JAX wrapper for traced functions.
+        return True
+    else:
       break
+
     fn_ = fn_.__wrapped__
   return False
 
@@ -95,10 +103,10 @@ def _assert_leaves_all_eq_comparator(
     error_msg_fn: Callable[[TLeaf, TLeaf, str, int, int],
                            str], path: Sequence[Any], *leaves: Sequence[TLeaf]):
   """Asserts all leaves are equal using custom comparator."""
-  path_ = "/".join(map(str, path))
+  path_str = "/".join(str(p) for p in path)
   for i in range(1, len(leaves)):
     if not equality_comparator(leaves[0], leaves[i]):
-      raise AssertionError(error_msg_fn(leaves[0], leaves[i], path_, 0, i))
+      raise AssertionError(error_msg_fn(leaves[0], leaves[i], path_str, 0, i))
 
 
 _TRACE_COUNTER = collections.Counter()
@@ -244,7 +252,7 @@ def assert_equal_shape(inputs: Sequence[Array]):
   Raises:
     AssertionError: if the shapes of all arrays do not match.
   """
-  if isinstance(inputs, Sequence):
+  if isinstance(inputs, collections.abc.Sequence):
     shape = inputs[0].shape
     expected_shapes = [shape] * len(inputs)
     shapes = [x.shape for x in inputs]
@@ -319,7 +327,7 @@ def assert_shape(inputs: Union[Scalar, Union[Array, Sequence[Array]]],
         "Expected shapes should be a list or tuple of ints.")
 
   # Ensure inputs and expected shapes are sequences.
-  if not isinstance(inputs, Sequence):
+  if not isinstance(inputs, collections.abc.Sequence):
     inputs = [inputs]
 
   # Shapes are always lists or tuples, not scalars.
@@ -375,7 +383,7 @@ def assert_equal_rank(inputs: Sequence[Array]):
   Raises:
     AssertionError: if the ranks of all arrays do not match.
   """
-  if isinstance(inputs, Sequence):
+  if isinstance(inputs, collections.abc.Sequence):
     rank = len(inputs[0].shape)
     expected_ranks = [rank] * len(inputs)
     ranks = [len(x.shape) for x in inputs]
@@ -416,10 +424,10 @@ def assert_rank(
                      "be a collection of integers but was an array.")
 
   # Ensure inputs and expected ranks are sequences.
-  if not isinstance(inputs, Sequence):
+  if not isinstance(inputs, collections.abc.Sequence):
     inputs = [inputs]
-  if (not isinstance(expected_ranks, Sequence) or
-      isinstance(expected_ranks, Set)):
+  if (not isinstance(expected_ranks, collections.abc.Sequence) or
+      isinstance(expected_ranks, collections.abc.Set)):
     expected_ranks = [expected_ranks] * len(inputs)
   if len(inputs) != len(expected_ranks):
     raise AssertionError(
@@ -437,12 +445,13 @@ def assert_rank(
     # Multiple expected options can be specified.
 
     # Check against old usage where options could be any sequence
-    if isinstance(expected, Sequence) and not isinstance(expected, Set):
-      raise ValueError(
-          "Error in rank compatibility check: "
-          "Expected ranks should be integers or sets of integers.")
+    if (isinstance(expected, collections.abc.Sequence) and
+        not isinstance(expected, collections.abc.Set)):
+      raise ValueError("Error in rank compatibility check: "
+                       "Expected ranks should be integers or sets of integers.")
 
-    options = expected if isinstance(expected, Set) else {expected}
+    options = (
+        expected if isinstance(expected, collections.abc.Set) else {expected})
 
     if rank not in options:
       errors.append((idx, rank, shape, expected))
@@ -465,6 +474,7 @@ def assert_type(
   ```
     assert_type(7, int)
     assert_type(7.1, float)
+    assert_type(False, bool)
     assert_type([7, 8], int)
     assert_type([7, 7.1], [int, float])
     assert_type(np.array(7), int)
@@ -491,14 +501,15 @@ def assert_type(
 
   errors = []
   if len(inputs) != len(expected_types):
-    raise AssertionError(
-        "Length of `inputs` and `expected_types` must match: `inputs` has length "
-        f"{len(inputs)}, `expected_types` has length {len(expected_types)}.")
+    raise AssertionError(f"Length of `inputs` and `expected_types` must match, "
+                         f"got {len(inputs)} != {len(expected_types)}.")
   for idx, (x, expected) in enumerate(zip(inputs, expected_types)):
     if jnp.issubdtype(expected, jnp.floating):
       parent = jnp.floating
     elif jnp.issubdtype(expected, jnp.integer):
       parent = jnp.integer
+    elif jnp.issubdtype(expected, jnp.bool_):
+      parent = jnp.bool_
     else:
       raise AssertionError(
           f"Error in type compatibility check, unsupported dtype '{expected}'.")
@@ -526,7 +537,7 @@ def assert_axis_dimension(tensor: Array, axis: int, expected: int):
       prescribed value.
   """
   tensor = jnp.asarray(tensor)
-  if axis >= len(tensor.shape):
+  if axis >= len(tensor.shape) or axis < -len(tensor.shape):
     raise AssertionError(
         f"Expected tensor to have dim '{expected}' along axis '{axis}' but"
         f" axis '{axis}' not available: tensor rank is '{len(tensor.shape)}'.")
@@ -548,18 +559,18 @@ def assert_axis_dimension_gt(tensor: Array, axis: int, val: int):
     AssertionError: if the dimension of `axis` is not greater than `val`.
   """
   tensor = jnp.asarray(tensor)
-  if axis >= len(tensor.shape):
+  if axis >= len(tensor.shape) or axis < -len(tensor.shape):
     raise AssertionError(
         f"Expected tensor to have dim greater than '{val}' on axis '{axis}' but"
         f" axis '{axis}' not available: tensor rank is '{len(tensor.shape)}'.")
   if tensor.shape[axis] <= val:
     raise AssertionError(
-        f"Expected tensor to have dim greater than '{val}' on axis '{axis}'"
-        f" but got '{tensor.shape[axis]}' instead.")
+        f"Expected tensor to have dimension greater than '{val}' on axis"
+        f" '{axis}' but got '{tensor.shape[axis]}' instead.")
 
 
 def assert_numerical_grads(
-    f: Callable[[Sequence[Array]], Array],
+    f: Callable[..., Array],
     f_args: Sequence[Array],
     order: int,
     atol: float = 0.01,
@@ -587,8 +598,58 @@ def assert_numerical_grads(
     jax_test.check_grads(f, f_args, order=order, atol=atol, **check_kwargs)
 
 
+def assert_tree_no_nones(tree: ArrayTree):
+  """Asserts tree does not contain `None`s.
+
+  Args:
+    tree: a tree to assert.
+
+  Raises:
+    AssertionError: if the tree contains `None`s.
+  """
+
+  def _assert_fn(path, leaf):
+    if leaf is None:
+      formatted_path = "/".join(str(p) for p in path)
+      raise AssertionError(f"`None` detected at '{formatted_path}'.")
+
+  dm_tree.map_structure_with_path(_assert_fn, tree)
+
+
+def assert_tree_shape_prefix(tree: ArrayTree,
+                             shape_prefix: Sequence[int],
+                             *,
+                             ignore_nones: bool = False):
+  """Asserts all tree leaves' shapes have the same prefix.
+
+  Args:
+    tree: a tree to assert.
+    shape_prefix: an expected shapes' prefix.
+    ignore_nones: whether to ignore `None`s in the tree.
+  Raise:
+    AssertionError: if some leaf's shape doesn't start with the expected prefix;
+                    if `ignore_nones` isn't set and the tree contains `None`s.
+  """
+
+  if not ignore_nones:
+    assert_tree_no_nones(tree)
+
+  def _assert_fn(path, leaf):
+    if leaf is None: return
+
+    prefix = leaf.shape[:len(shape_prefix)]
+    if prefix != shape_prefix:
+      raise AssertionError(
+          f"Tree leaf '{'/'.join(path)}' has a shape prefix "
+          f"diffent from expected: {prefix} != {shape_prefix}.")
+
+  dm_tree.map_structure_with_path(_assert_fn, tree)
+
+
 def assert_tree_all_equal_structs(*trees: Sequence[ArrayTree]):
-  """Assert trees have the same structure.
+  """Asserts trees have the same structure.
+
+  Note that `None`s are treated as PyTree nodes.
 
   Args:
     *trees: trees which structures to assert on equality.
@@ -597,7 +658,7 @@ def assert_tree_all_equal_structs(*trees: Sequence[ArrayTree]):
     AssertionError: if structures of any two trees are different.
   """
   first_treedef = jax.tree_structure(trees[0])
-  other_treedefs = map(jax.tree_structure, trees[1:])
+  other_treedefs = (jax.tree_structure(t) for t in trees[1:])
   for i, treedef in enumerate(other_treedefs, start=1):
     if first_treedef != treedef:
       raise AssertionError(
@@ -608,25 +669,49 @@ def assert_tree_all_equal_structs(*trees: Sequence[ArrayTree]):
 
 def assert_tree_all_equal_comparator(equality_comparator: TLeavesEqCmpFn,
                                      error_msg_fn: TLeavesEqCmpErrorFn,
-                                     *trees: Sequence[ArrayTree]):
-  """Assert all trees are equal using custom comparator for leaves."""
-  if len(trees) < 2:
-    return
-  assert_tree_all_equal_structs(*trees)
+                                     *trees: Sequence[ArrayTree],
+                                     ignore_nones: bool = False):
+  """Asserts all trees are equal as per the custom comparator for leaves.
 
-  def _tree_error_msg_fn(l_1: TLeaf, l_2: TLeaf, path: str, i_1: int, i_2: int):
+  Args:
+    equality_comparator: a custom function that accepts two leaves and checks
+                         whether they are equal. Expected to be transitive.
+    error_msg_fn: a function accepting two unequal as per `equality_comparator`
+                  leaves and returning an error message.
+    *trees: trees to check on equality as per `equality_comparator`.
+    ignore_nones: whether to ignore `None`s in the trees.
+
+  Raises:
+    AssertionError: if `equality_comparator` returns False on any pair of trees.
+  """
+  if len(trees) < 2: return
+  assert_tree_all_equal_structs(*trees)
+  if not ignore_nones: assert_tree_no_nones(trees)
+
+  def tree_error_msg_fn(l_1: TLeaf, l_2: TLeaf, path: str, i_1: int, i_2: int):
     msg = error_msg_fn(l_1, l_2)
     return f"Trees {i_1} and {i_2} differ in leaves '{path}': {msg}."
 
+  def wrapped_equality_comparator(leaf_1, leaf_2):
+    if leaf_1 is None or leaf_1 is None:
+      # Either both or none of leaves can be `None`s.
+      assert leaf_1 is None and leaf_2 is None, (
+          "non-mutual cases must be caught by assert_tree_all_equal_structs")
+      if ignore_nones:
+        return True
+
+    return equality_comparator(leaf_1, leaf_2)
+
   cmp = functools.partial(_assert_leaves_all_eq_comparator,
-                          equality_comparator, _tree_error_msg_fn)
-  tree.map_structure_with_path(cmp, *trees)
+                          wrapped_equality_comparator, tree_error_msg_fn)
+  dm_tree.map_structure_with_path(cmp, *trees)
 
 
 def assert_tree_all_close(*trees: Sequence[ArrayTree],
                           rtol: float = 1e-07,
-                          atol: float = .0):
-  """Assert trees have leaves with approximately equal values.
+                          atol: float = .0,
+                          ignore_nones: bool = False):
+  """Asserts trees have leaves with approximately equal values.
 
   This compares the difference between values of actual and desired to
    atol + rtol * abs(desired).
@@ -635,30 +720,41 @@ def assert_tree_all_close(*trees: Sequence[ArrayTree],
     *trees: a sequence of trees with array leaves.
     rtol: relative tolerance.
     atol: absolute tolerance.
+    ignore_nones: whether to ignore `None`s in the trees.
+
   Raise:
     AssertionError: if the leaf values actual and desired are not equal up to
-      specified tolerance.
+      specified tolerance, or trees contain `None`s (with `ignore_nones=False`).
   """
   assert_fn = functools.partial(
       np.testing.assert_allclose, rtol=rtol, atol=atol,
       err_msg="Error in value equality check: Values not approximately equal")
-  cmp_fn = lambda arr_1, arr_2: bool(assert_fn(arr_1, arr_2) is None)
-  err_msg_fn = lambda arr_1, arr_2: None
-  assert_tree_all_equal_comparator(cmp_fn, err_msg_fn, *trees)
+
+  def cmp_fn(arr_1, arr_2):
+    assert_fn(arr_1, arr_2)  # Raises an AssertionError if values are not close.
+    return True
+
+  dummy_err_msg_fn = lambda arr_1, arr_2: None
+  assert_tree_all_equal_comparator(
+      cmp_fn, dummy_err_msg_fn, *trees, ignore_nones=ignore_nones)
 
 
-def assert_tree_all_equal_shapes(*trees: Sequence[ArrayTree]):
-  """Assert trees have the same structure and leaves' shapes.
+def assert_tree_all_equal_shapes(*trees: Sequence[ArrayTree],
+                                 ignore_nones: bool = False):
+  """Asserts trees have the same structure and leaves' shapes.
 
   Args:
     *trees: a sequence of trees with array leaves.
+    ignore_nones: whether to ignore `None`s in the trees.
 
-  Raise:
-    AssertionError: if trees' structures or leaves' shapes are different.
+  Raises:
+    AssertionError: if trees' structures or leaves' shapes are different;
+                    if trees contain `None`s (with `ignore_nones=False`).
   """
   cmp_fn = lambda arr_1, arr_2: arr_1.shape == arr_2.shape
   err_msg_fn = lambda arr_1, arr_2: f"shapes: {arr_1.shape} != {arr_2.shape}"
-  assert_tree_all_equal_comparator(cmp_fn, err_msg_fn, *trees)
+  assert_tree_all_equal_comparator(
+      cmp_fn, err_msg_fn, *trees, ignore_nones=ignore_nones)
 
 
 def assert_devices_available(
