@@ -74,14 +74,15 @@ class AssertMaxTracesTest(variants.TestCase):
     asserts.clear_trace_counter()
 
   def _init(self, fn_, init_type, max_traces, kwargs, static_arg):
-    variant_kwargs = dict()
+    """Initializes common test cases."""
+    variant_kwargs = {}
     if static_arg:
       variant_kwargs['static_argnums'] = 1
 
     if kwargs:
-      args, kwargs = list(), dict(n=max_traces)
+      args, kwargs = [], {'n': max_traces}
     else:
-      args, kwargs = [max_traces], dict()
+      args, kwargs = [max_traces], {}
 
     if init_type == 't1':
 
@@ -394,13 +395,15 @@ class RankAssertTest(parameterized.TestCase):
       asserts.assert_rank(rank_array(2), np.array([2]))
 
   def test_rank_should_fail_wrong_expectation_structure(self):
+    # pytype: disable=wrong-arg-types
     with self.assertRaisesRegex(  # pylint: disable=g-error-prone-assert-raises
         ValueError, 'Expected ranks should be integers or sets of integers'):
-      asserts.assert_rank(rank_array(2), [[1, 2]])  # pytype: disable=wrong-arg-types
+      asserts.assert_rank(rank_array(2), [[1, 2]])
 
     with self.assertRaisesRegex(  # pylint: disable=g-error-prone-assert-raises
         ValueError, 'Expected ranks should be integers or sets of integers'):
-      asserts.assert_rank([rank_array(1), rank_array(2)], [[1], [2]])  # pytype: disable=wrong-arg-types
+      asserts.assert_rank([rank_array(1), rank_array(2)], [[1], [2]])
+    # pytype: enable=wrong-arg-types
 
   @parameterized.named_parameters(
       ('rank_1', rank_array(1), 2),
@@ -470,6 +473,10 @@ class TypeAssertTest(parameterized.TestCase):
       ('one_int', 3, float),
       ('many_floats', [1., 2., 3.], int),
       ('many_floats_verbose', [1., 2., 3.], [float, float, int]),
+      ('one_bool_as_float', True, float),
+      ('one_bool_as_int', True, int),
+      ('one_float_as_bool', 3., bool),
+      ('one_int_as_bool', 3, bool),
   )
   def test_type_should_fail_scalar(self, scalars, wrong_type):
     with self.assertRaisesRegex(AssertionError,
@@ -490,6 +497,7 @@ class TypeAssertTest(parameterized.TestCase):
   @parameterized.named_parameters(
       ('one_float', 3., float),
       ('one_int', 3, int),
+      ('one_bool', True, bool),
       ('many_floats', [1., 2., 3.], float),
       ('many_floats_verbose', [1., 2., 3.], [float, float, float]),
   )
@@ -529,7 +537,7 @@ class TypeAssertTest(parameterized.TestCase):
   )
   def test_type_should_fail_wrong_length(self, array, wrong_type):
     with self.assertRaisesRegex(
-        AssertionError, 'Length of `inputs` and `expected_types` must match:'):
+        AssertionError, 'Length of `inputs` and `expected_types` must match'):
       asserts.assert_type(array, wrong_type)
 
   def test_type_should_fail_unsupported_dtype(self):
@@ -546,31 +554,57 @@ class AxisDimensionAssertionsTest(parameterized.TestCase):
 
   def test_assert_axis_dimension_pass(self):
     tensor = jnp.ones((3, 2, 7, 2))
-    for i, s in enumerate(tensor.shape):
+    for i in range(-tensor.ndim, tensor.ndim):
+      s = tensor.shape[i]
       asserts.assert_axis_dimension(tensor, axis=i, expected=s)
 
   def test_assert_axis_dimension_fail(self):
     tensor = jnp.ones((3, 2, 7, 2))
-    for i, s in enumerate(tensor.shape):
+    for i in range(-tensor.ndim, tensor.ndim):
+      s = tensor.shape[i]
       with self.assertRaisesRegex(
           AssertionError, 'Expected tensor to have dimension'):
         asserts.assert_axis_dimension(tensor, axis=i, expected=s+1)
 
-  def test_assert_axis_invalid(self):
+  def test_assert_axis_dimension_axis_invalid(self):
     tensor = jnp.ones((3, 2))
-    with self.assertRaisesRegex(AssertionError, 'not available'):
-      asserts.assert_axis_dimension(tensor, axis=2, expected=1)
+    for i in (2, -3):
+      with self.assertRaisesRegex(AssertionError, 'not available'):
+        asserts.assert_axis_dimension(tensor, axis=i, expected=1)
+
+  def test_assert_axis_dimension_gt_pass(self):
+    tensor = jnp.ones((3, 2, 7, 2))
+    for i in range(-tensor.ndim, tensor.ndim):
+      s = tensor.shape[i]
+      asserts.assert_axis_dimension_gt(tensor, axis=i, val=s-1)
+
+  def test_assert_axis_dimension_gt_fail(self):
+    tensor = jnp.ones((3, 2, 7, 2))
+    for i in range(-tensor.ndim, tensor.ndim):
+      s = tensor.shape[i]
+      with self.assertRaisesRegex(
+          AssertionError, 'Expected tensor to have dimension greater than'):
+        asserts.assert_axis_dimension_gt(tensor, axis=i, val=s)
+
+  def test_assert_axis_dimension_gt_axis_invalid(self):
+    tensor = jnp.ones((3, 2))
+    for i in (2, -3):
+      with self.assertRaisesRegex(AssertionError, 'not available'):
+        asserts.assert_axis_dimension_gt(tensor, axis=i, val=0)
 
 
 class TreeAssertionsTest(parameterized.TestCase):
 
   def _assert_tree_structs_validation(self, assert_fn):
+    """Checks that assert_fn correctly processes invalid args' structs."""
+
     get_val = lambda: jnp.zeros([3])
     tree1 = [[get_val(), get_val()], get_val()]
     tree2 = [[get_val(), get_val()], get_val()]
     tree3 = [get_val(), [get_val(), get_val()]]
     tree4 = [get_val(), [get_val(), get_val()], get_val()]
     tree5 = dict(x=1, y=2, z=3)
+    tree6 = dict(x=1, y=2, z=3, n=None)
 
     with self.assertRaisesRegex(
         AssertionError, 'Error in tree structs equality check.*trees 0 and 1'):
@@ -592,6 +626,11 @@ class TreeAssertionsTest(parameterized.TestCase):
         AssertionError, 'Error in tree structs equality check.*trees 0 and 2'):
       assert_fn(tree2, tree1, tree4)
 
+    # Test `None`s.
+    with self.assertRaisesRegex(
+        AssertionError, 'Error in tree structs equality check.*trees 0 and 1'):
+      assert_fn(tree5, tree6)
+
   def test_tree_all_finite_passes_finite(self):
     finite_tree = {'a': jnp.ones((3,)), 'b': jnp.array([0.0, 0.0])}
     asserts.assert_tree_all_finite(finite_tree)
@@ -622,6 +661,30 @@ class TreeAssertionsTest(parameterized.TestCase):
     tree2 = (jnp.array([1.0, 1.0 + 1e-9]),)
     asserts.assert_tree_all_close(tree1, tree2)
 
+  def test_assert_tree_all_close_nones(self):
+    tree = {'a': [jnp.zeros((1,))], 'b': None}
+    asserts.assert_tree_all_close(tree, tree, ignore_nones=True)
+    with self.assertRaisesRegex(AssertionError, '`None` detected'):
+      asserts.assert_tree_all_close(tree, tree, ignore_nones=False)
+
+  def test_assert_tree_all_equal_shapes_nones(self):
+    tree = {'a': [jnp.zeros((1,))], 'b': None}
+    asserts.assert_tree_all_equal_shapes(tree, tree, ignore_nones=True)
+    with self.assertRaisesRegex(AssertionError, '`None` detected'):
+      asserts.assert_tree_all_equal_shapes(tree, tree, ignore_nones=False)
+
+  def test_assert_tree_no_nones(self):
+    tree_ok = {'a': [jnp.zeros((1,))], 'b': 1}
+    asserts.assert_tree_no_nones(tree_ok)
+
+    tree_with_none = {'a': [jnp.zeros((1,))], 'b': None}
+    with self.assertRaisesRegex(AssertionError, '`None` detected'):
+      asserts.assert_tree_no_nones(tree_with_none)
+
+    # Check `None`.
+    with self.assertRaisesRegex(AssertionError, '`None` detected'):
+      asserts.assert_tree_no_nones(None)
+
   def test_assert_tree_all_close_fails_different_structure(self):
     self._assert_tree_structs_validation(asserts.assert_tree_all_close)
 
@@ -668,6 +731,25 @@ class TreeAssertionsTest(parameterized.TestCase):
     asserts.assert_tree_all_equal_structs(tree3, tree3)
     self._assert_tree_structs_validation(asserts.assert_tree_all_equal_structs)
 
+  def test_assert_tree_shape_prefix(self):
+    tree = {'x': {'y': np.zeros([3, 2])}, 'z': np.zeros([3, 2, 1])}
+    asserts.assert_tree_shape_prefix(tree, ())
+    asserts.assert_tree_shape_prefix(tree, (3,))
+    asserts.assert_tree_shape_prefix(tree, (3, 2))
+
+    with self.assertRaisesRegex(
+        AssertionError,
+        r'Tree leaf \'x/y\' .* diffent from expected: \(3, 2\) != \(3, 2, 1\)'
+    ):
+      asserts.assert_tree_shape_prefix(tree, (3, 2, 1))
+
+  def test_assert_tree_shape_prefix_none(self):
+    tree = {'x': np.zeros([3]), 'n': None}
+    asserts.assert_tree_shape_prefix(tree, (3,), ignore_nones=True)
+
+    with self.assertRaisesRegex(AssertionError, '`None` detected'):
+      asserts.assert_tree_shape_prefix(tree, (3,), ignore_nones=False)
+
 
 class NumDevicesAssertTest(parameterized.TestCase):
 
