@@ -37,7 +37,6 @@ import numpy as np
 
 FLAGS = flags.FLAGS
 
-ArrayDevice = pytypes.ArrayDevice
 ArrayBatched = pytypes.ArrayBatched
 
 DEFAULT_FN = lambda arg_0, arg_1: arg_1 - arg_0
@@ -241,7 +240,7 @@ class ParamsProductTest(absltest.TestCase):
     self.assertEqual(named_product, named_expected)
 
 
-class FailedTestsTest(variants.TestCase):
+class FailedTestsTest(absltest.TestCase):
   # Inner class prevents FailedTest being run by `absltest.main()`.
 
   class FailedTest(variants.TestCase):
@@ -252,8 +251,7 @@ class FailedTestsTest(variants.TestCase):
 
     @variants.variants(without_jit=True)
     def test_error(self):
-      msg = 'this message does not specify the Chex variant'
-      raise ValueError(msg)
+      raise ValueError('this message does not specify the Chex variant')
 
   def setUp(self):
     super().setUp()
@@ -285,7 +283,7 @@ class OneFailedVariantTest(variants.TestCase):
 
       @self.variant
       def fails_for_without_device_variant(x):
-        self.assertIsInstance(x, ArrayDevice)
+        self.assertIsInstance(x, jnp.DeviceArray)
 
       fails_for_without_device_variant(42)
 
@@ -327,6 +325,7 @@ class BaseClassesTest(parameterized.TestCase):
   """Tests different combinations of base classes for a variants test."""
 
   def generate_test_class(self, base_1, base_2):
+    """Returns a test class derived from the specified bases."""
 
     class InnerBaseClassTest(base_1, base_2):
 
@@ -630,8 +629,8 @@ class WithJitTest(parameterized.TestCase):
 
 def _test_fn_without_device(arg_0, arg_1):
   tc = unittest.TestCase()
-  tc.assertNotIsInstance(arg_0, ArrayDevice)
-  tc.assertNotIsInstance(arg_1, ArrayDevice)
+  tc.assertNotIsInstance(arg_0, jnp.DeviceArray)
+  tc.assertNotIsInstance(arg_1, jnp.DeviceArray)
   return DEFAULT_FN(arg_0, arg_1)
 
 
@@ -652,8 +651,8 @@ class WithoutDeviceTest(parameterized.TestCase):
 
 def _test_fn_with_device(arg_0, arg_1):
   tc = unittest.TestCase()
-  tc.assertIsInstance(arg_0, ArrayDevice)
-  tc.assertIsInstance(arg_1, ArrayDevice)
+  tc.assertIsInstance(arg_0, jnp.DeviceArray)
+  tc.assertIsInstance(arg_1, jnp.DeviceArray)
   return DEFAULT_FN(arg_0, arg_1)
 
 
@@ -679,7 +678,7 @@ class WithDeviceTest(parameterized.TestCase):
     @self.variant(ignore_argnums=(0, 2))
     def fn(arg_0, arg_1, float_arg):
       self.assertIsInstance(arg_0, static_type)
-      self.assertIsInstance(arg_1, ArrayDevice)
+      self.assertIsInstance(arg_1, jnp.DeviceArray)
       self.assertIsInstance(float_arg, float)
       return DEFAULT_FN(arg_0, arg_1)
 
